/*
 * $Id$
 *
 * Serial Wire Debug Open API. Distributed on BSD License.
 * Library Header File.
 *
 * Copyright (C) 2010, Tomasz Boleslaw CEDRO (http://www.tomek.cedro.info)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the Tomasz Boleslaw CEDRO nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.*
 *
 * Written by Tomasz Boleslaw CEDRO <tomek.cedro@gmail.com>, 2010;
 *
 */


#ifndef __LIBSWD_H__
#define __LIBSWD_H__

/* SWD Packets Bit Fields and Values */
#define SWD_REQUEST_START_BITNUM  7  // Packet Start bit, always 1
#define SWD_REQUEST_APnDP_BITNUM  6  // Access Port (high) or Debug Port (low) access
#define SWD_REQ_RnW_BITNUM    5  // Read (high) or Write (low) access
#define SWD_REQUEST_A2_BITNUM     4  // Register Address bit 2
#define SWD_REQUEST_A3_BITNUM     3  // Register Address bit 3
#define SWD_REQUEST_PARITY_BITNUM 2  // Odd Parity calculated from APnDP, RnW, A[2:3]
#define SWD_REQUEST_STOP_BITNUM   1  // Packet Stop bit, always 0
#define SWD_REQUEST_PARK_BITNUM   0  // Park wire and switch between receive/transmit

#define SWD_REQUEST_START_VAL  1  // Start Bit is always 1
#define SWD_REQUEST_STOP_VAL   0  // Stop Bit is always 0
#define SWD_REQUEST_PARK_VAL   1  // Park bus and put outputs into Hi-Z state
#define SWD_REQUEST_BITLEN     8


#define SWD_ACK_BITLEN     3
#define SWD_ACK_OK         0b100
#define SWD_ACK_WAIT       0b010
#define SWD_ACK_FAULT      0b001

#define SWD_DP_ADDR_IDCODE   0b00 // RO
#define SWD_DP_ADDR_ABORT    0b00 // WO
#define SWD_DP_ADDR_CRTLSTAT 0b01 // R/W, CTRLSEL=b0
#define SWD_DP_ADDR_WCR      0b01 // R/W, CTRLSEL=b1
#define SWD_DP_ADDR_RESEND   0b10 // RO
#define SWD_DP_ADDR_SELECT   0b10 // WO
#define SWD_DP_ADDR_RDBUF    0b11 // RO

/* SW-DP ABORT Register Bitmask */
#define SWD_ABORT_BITNUM_DAPABORT   0
#define SWD_ABORT_BITNUM_DSTKCMPCLR  1
#define SWD_ABORT_BITNUM_DSTKERRCLR  2
#define SWD_ABORT_BITNUM_DWDERRCLR   3
#define SWD_ABORT_BITNUM_DORUNERRCLR 4

/* SW-DP CTRL/STAT Register Bitmask and Control Values */
#define SWD_CTRLSTAT_BITNUM_ORUNDETECT     0
#define SWD_CTRLSTAT_BITNUM_OSTICKYORUN    1
#define SWD_CTRLSTAT_BITNUM_OTRNMODE       2
#define SWD_CTRLSTAT_BITNUM_OSTICKYCMP     4
#define SWD_CTRLSTAT_BITNUM_OSTICKYERR     5
#define SWD_CTRLSTAT_BITNUM_OREADOK        6
#define SWD_CTRLSTAT_BITNUM_OWDATAERR      7
#define SWD_CTRLSTAT_BITNUM_OMASKLANE      8
#define SWD_CTRLSTAT_BITNUM_OTRNCNT        12
#define SWD_CTRLSTAT_BITNUM_OCDBGRSTREQ    26
#define SWD_CTRLSTAT_BITNUM_OCDBGRSTACK    27
#define SWD_CTRLSTAT_BITNUM_OCDBGPWRUPREQ  28
#define SWD_CTRLSTAT_BITNUM_OCDBGPWRUPACK  29
#define SWD_CTRLSTAT_BITNUM_OCSYSPWRUPREQ  30
#define SWD_CTRLSTAT_BITNUM_OCSYSPWRUPACK  31

#define SWD_MASKLANE_0 0b0001  // Compare byte lane 0 (0x------FF)
#define SWD_MASKLANE_1 0b0010  // Compare byte lane 1 (0x----FF--)
#define SWD_MASKLANE_2 0b0100  // Compare byte lane 2 (0x--FF----)
#define SWD_MASKLANE_3 0b1000  // Compare byte lane 3 (0xFF------)

/* SW-DP SELECT Register Bitmask */
#define SWD_SELECT_BITNUM_CTRLSEL   0
#define SWD_SELECT_BITNUM_APBANKSEL 4 
#define SWD_SELECT_BITNUM_APSEL     24

/* SW-DP WCR Register Bitmask and Control Values */
#define SWD_WCR_BITNUM_PRESCALER  0
#define SWD_WCR_BITNUM_WIREMODE   6
#define SWD_WCR_BITNUM_TURNROUND  8

#define SWD_TURNROUND_1 0b00
#define SWD_TURNROUND_2 0b01
#define SWD_TURNROUND_3 0b10
#define SWD_TURNROUND_4 0b11
#define SWD_TURNROUND_MIN SWD_TURNROUND_1
#define SWD_TURNROUND_MAX SWD_TURNROUND_4

/* AHB-AP Registers Addresses */
#define AHB_AP_CONTROLSTATUS 0x00  // R/W, 32bit, reset value: 0x43800042 
#define AHB_AP_TAR           0x04  // R/W, 32bit, reset value: 0x00000000
#define AHB_AP_DRW           0x0C  // R/W, 32bit
#define AHB_AP_BD0           0x10  // R/W, 32bit
#define AHB_AP_BD1           0x14  // R/W, 32bit
#define AHB_AP_BD2           0x18  // R/W, 32bit
#define AHB_AP_BD3           0x1C  // R/W, 32bit
#define AHB_AP_DROMT         0xF8  // RO, 32bit, reset value: 0xE00FF000
#define AHB_AP_IDR           0xFC  // RO, 32bit, reset value: 0x24770001

/* Static commands payload. */
#define SWD_CMD_SWDPRESET static char[8] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff} 
#define SWD_CMD_JTAG2SWD static char[2] = {0x79, 0xe7}
#define SWD_CMD_SWD2JTAG static char[2] = {0x3c, 0xe7}

/* Status and Error definitions */

typedef enum {
 SWD_OK                = 0,
 SWD_ERROR             =-1,
 SWD_ERROR_NULLPOINTER =-2,
 SWD_ERROR_NULLQUEUE   =-3,
 SWD_ERROR_NULLTRN     =-4,
 SWD_ERROR_BADPARAM    =-5,
 SWD_ERROR_OUTOFMEM    =-6,
 SWD_ERROR_BADRESULT   =-7,
 SWD_ERROR_OUTOFRANGE  =-8,
 SWD_ERROR_DEFINITION  =-9,
 SWD_ERROR_NULLCONTEXT =-10,
 SWD_ERROR_QUEUE       =-11
} swd_error_t;


#define SWD_DATA_MAXBITCOUNT   32
#define SWD_DATA_BYTESIZE      8

/* Command types, read > 0, write < 0, dir change = 0. */
typedef enum SWD_CMDTYPE {
 SWD_CMDTYPE_PARMOSI=-4,  // master output parity data
 SWD_CMDTYPE_BITMOSI=-3,  // single bit slave to master data
 SWD_CMDTYPE_CONTROL=-2, // control sequence byte (ie. swdpreset)
 SWD_CMDTYPE_MOSI=-1,   // data chunk from slave to master
 SWD_CMDTYPE_TRN=0,     // turnaround and switch between Tx/Rx
 SWD_CMDTYPE_MISO=1,    // data chunk from master to slave
 SWD_CMDTYPE_ACK=2,     // slave acknowledgement
 SWD_CMDTYPE_BITMISO=3,   // single bit master to slave data
 SWD_CMDTYPE_PARMISO=4    // slave output parity data
} swd_cmdtype_t;

typedef struct swd_cmd_t {
 union {          // payload data union
  char RnW;       // turnaround buffer control value 
  char request;   // request header data
  char ack;       // acknowledge response from target
  int misodata;   // data read from target
  int mosidata;   // data written to target
  char parity;    // parity bit for data
  char control;
 };
 char bits;       // payload bit count = clk pulses
 char type;       // operation type as defined by SWD_CMDTYPE_*
 char done;       // non zero if operation already executed
 struct swd_cmd_t *prev, *next;
} swd_cmd_t;


typedef struct {
 char trnlen;
} swd_params_t;

typedef struct {
 char ack;
 int idcode;
 int abort;
 int ctrlstat;
 int wcr;
 int select;
 int rdbuf;
} swd_swdp_t;

typedef struct {
 int controlstatus;
 int tar;
 int drw;
 int bd0;
 int bd1;
 int bd2;
 int bd3;
 int dromt;
 int idr;
} swd_ahbap_t;

/* SWD context keeps all the information about the target status,
 * including data and instruction registers, also the command queue.
 * Writes and reads from the target should be made only with use of context,
 * otherwise desynchronization with the target may occur.
 */
typedef struct {
 swd_cmd_t *cmdq;
 swd_params_t params;
 swd_swdp_t rswdp;
 swd_swdp_t wswdp;
 swd_ahbap_t rahbap;
 swd_ahbap_t wahbap;
} swd_ctx_t;

/* These functions pointers use cable driver to switch buffers into Tx/Rx.*/
int swd_trn_hosttx();
int swd_trn_hostrx();


#endif

